@@ -27,6 +27,9 @@ class Model(nn.Module):
 	def build(self, *inputs):
 		pass 
 
+	def build_forward(self, *inputs, **kwargs):
+		return self.forward(*inputs, **kwargs)
+
 	def __call__(self, *input, **kwargs):
 		if not self.is_built:
 			self.build(*input)
@@ -39,7 +42,10 @@ class Model(nn.Module):
 		if torch._C._get_tracing_state():
 			result = self._slow_forward(*input, **kwargs)
 		else:
-			result = self.forward(*input, **kwargs)
+			if not self.is_built:
+				result = self.build_forward(*input, **kwargs)
+			else:
+				result = self.forward(*input, **kwargs)
 		for hook in self._forward_hooks.values():
 			hook_result = hook(self, input, result)
 			if hook_result is not None:
@@ -129,6 +135,59 @@ class conv2D(Model):
 	def forward(self, x):
 		return F.conv2d(x, self.weight, self.bias, self.stride, self.pad, self.dilation_rate, self.gropus)
 
+class deconv2D(Model):
+	def initialize(self, size, outchn, stride=1, pad='SAME_LEFT', dilation_rate=1, usebias=True, gropus=1):
+		self.size = size
+		self.outchn = outchn
+		self.stride = stride
+		self.usebias = usebias
+		self.gropus = gropus
+		self.dilation_rate = dilation_rate
+		assert (pad in ['VALID','SAME_LEFT'])
+		self.pad = pad 
+		self.padmethod = pad
+
+	def _parse_args(self, input_shape):
+		inchannel = input_shape[1]
+		# parse args
+		if isinstance(self.size,int):
+			if self.pad == 'VALID':
+				self.pad = 0
+			else:
+				self.pad = (self.size + (self.dilation_rate-1) * ( self.size-1 ))//2 - (1 - self.size%2)
+				# self.pad = self.dilation_rate * (self.size - 1 ) 
+			self.out_pad = self.stride - 1
+			self.size = [inchannel, self.outchn // self.gropus, self.size, self.size]
+		else:
+			raise Exception("Deconv kernel only supports int")
+
+	def build(self, *inputs):
+		# print('building...')
+		inp = inputs[0]
+		self._parse_args(inp.shape)
+		self.weight = Parameter(torch.Tensor(*self.size))
+		if self.usebias:
+			self.bias = Parameter(torch.Tensor(self.outchn))
+		else:
+			self.register_parameter('bias', None)
+		self.reset_params()
+
+	def reset_params(self):
+		_resnet_normal(self.weight)
+		if self.bias is not None:
+			fan_in, _ = init._calculate_fan_in_and_fan_out(self.weight)
+			bound = 1 / math.sqrt(fan_in)
+			init.uniform_(self.bias, -bound, bound)
+
+	def forward(self, x):
+		inh, inw = x.shape[2], x.shape[3]
+		x = F.conv_transpose2d(x, self.weight, self.bias, self.stride, self.pad, self.out_pad, self.gropus, self.dilation_rate)
+		outh, outw = x.shape[2], x.shape[3]
+		if self.padmethod=='SAME_LEFT':
+			if outh!=inh*self.stride or outw!=inw*self.stride:
+				x = x[:,:,:inh*self.stride,:inw*self.stride]
+		return x 
+
 class dwconv2D(Model):
 	# depth-wise conv2d
 	def initialize(self, size, multiplier, stride=1, pad='SAME_LEFT', dilation_rate=1, usebias=True):
@@ -164,7 +223,7 @@ class dwconv2D(Model):
 		self._parse_args(inp.shape)
 		self.weight = Parameter(torch.Tensor(*self.size))
 		if self.usebias:
-			self.bias = Parameter(torch.Tensor(self.outchn))
+			self.bias = Parameter(torch.Tensor(self.size[0]))
 		else:
 			self.register_parameter('bias', None)
 		self.reset_params()
@@ -483,7 +542,7 @@ class NNUpSample(Model):
 		# print('building...')
 		inp = inputs[0]
 		self._parse_args(inp.shape)
-		self.weight = Parameter(torch.Tensor(*self.size))
+		self.weight = Parameter(torch.Tensor(*self.size), requires_grad=False)
 		self.register_parameter('bias', None)
 		self.reset_params()
 
@@ -491,7 +550,8 @@ class NNUpSample(Model):
 		init.ones_(self.weight)
 
 	def forward(self, x):
-		w = self.weight.detach()
+		# w = self.weight.detach()
+		w = self.weight
 		return F.conv_transpose2d(x, w, self.bias, self.scale, 0, 0, self.inchannel, 1)
 
 def activation(x, act, **kwargs):
@@ -522,14 +582,10 @@ class Activation(Model):
 			return activation(x, self.act)
 
 class graphConvLayer(Model):
-	def __init__(self, outsize, adj_mtx=None, adj_fn=None, values=None, usebias=True):
-		assert (adj_mtx is None) ^ (adj_fn is None), 'Assign either adj_mtx or adj_fn' 
+	def initialize(self, outsize, usebias=True, norm=True):
 		self.outsize = outsize
-		self.adj_mtx = adj_mtx
-		self.adj_fn = adj_fn
-		self.values = values
 		self.usebias = usebias
-		self.normalized = False
+		self.norm = norm 
 
 	def _parse_args(self, input_shape):
 		# set size
@@ -565,14 +621,12 @@ class graphConvLayer(Model):
 			A_ = torch.mm(A_, S)
 		return A_
 
-	def forward(self, x):
-		if self.adj_mtx is not None:
-			if not self.normalized:
-				self.adj_mtx = self._normalize_adj_mtx(self.adj_mtx)
-				self.normalized = True
-		else:
-			A = self.adj_fn(x)
-		res = torch.mm(A, x)
+	def forward(self, x, adj, affinity_grad=True):
+		if self.norm:
+			adj = self._normalize_adj_mtx(adj)
+		if not affinity_grad:
+			adj = adj.detach()
+		res = torch.mm(adj, x)
 		res = F.linear(res, self.weight, self.bias)
 		return res 
 	
