@@ -168,6 +168,75 @@ class ConvLayer(Model):
 			L.record_params.append(res)
 		return x 
 
+class DeConvLayer(Model):
+	def initialize(self, size, outchn, stride=1, pad='SAME_LEFT', dilation_rate=1, activation=-1, batch_norm=False, affine=True, usebias=True, groups=1):
+		self.conv = L.deconv2D(size, outchn, stride, pad, dilation_rate, usebias, groups)
+		if batch_norm:
+			self.bn = L.BatchNorm(affine=affine)
+		self.batch_norm = batch_norm
+		self.activation = activation
+		if self.activation == PARAM_PRELU:
+			self.act = torch.nn.PReLU(num_parameters=outchn)
+		elif self.activation==PARAM_PRELU1:
+			self.act = torch.nn.PReLU(num_parameters=1)
+	def forward(self, x):
+		if self._record:
+			record_flag = True
+			self.un_record()
+		else:
+			record_flag = False
+		# print('Record:',record_flag)
+		x = self.conv(x)
+		if self.batch_norm:
+			x = self.bn(x)
+		if self.activation==PARAM_PRELU or self.activation==PARAM_PRELU1:
+			x = self.act(x)
+		else:
+			x = L.activation(x, self.activation)
+
+		if record_flag:
+			# print(self._merge_bn)
+			# do record
+			if self._merge_bn:
+				miu = self.bn.running_mean
+				var = self.bn.running_var
+				gamma = self.bn.weight
+				beta = self.bn.bias 
+				eps = self.bn.eps 
+				if gamma is None:
+					gamma = 1 
+				if beta is None:
+					beta = 0
+				weight = self.conv.weight
+				bias = self.conv.bias
+
+				if bias is not None:
+					b = gamma * (bias - miu) / torch.sqrt(eps + var) + beta 
+				else:
+					b = beta - (gamma * miu / torch.sqrt(eps + var))
+				bn_w = gamma / torch.sqrt(eps + var)
+				bn_w = bn_w.unsqueeze(-1).unsqueeze(-1).unsqueeze(-1)
+				w = bn_w * weight
+
+				res = {}
+				base_name = list(self.named_parameters())[0][0]
+				base_name = '.'.join(base_name.split('.')[:-1])
+				res[base_name+'.weight'] = w 
+				res[base_name+'.bias'] = b
+
+				if self.activation==PARAM_PRELU or self.activation==PARAM_PRELU1:
+					actw = self.act.weight
+					res['act.weight'] = actw
+				# print(res.keys())
+			else:
+				res = {}
+				for p in self.named_parameters():
+					res[p[0]] = p[1]
+				for p in self.named_buffers():
+					res[p[0]] = p[1]
+			L.record_params.append(res)
+		return x 
+
 class DWConvLayer(Model):
 	def initialize(self, size, multiplier, stride=1, pad='SAME_LEFT', dilation_rate=1, activation=-1, batch_norm=False, affine=True, usebias=True):
 		self.conv = L.dwconv2D(size, multiplier, stride, pad, dilation_rate, usebias)
@@ -198,12 +267,45 @@ class DWConvLayer(Model):
 			x = L.activation(x, self.activation)
 
 		if record_flag:
+			# print(self._merge_bn)
 			# do record
-			res = {}
-			for p in self.named_parameters():
-				res[p[0]] = p[1]
-			for p in self.named_buffers():
-				res[p[0]] = p[1]
+			if self._merge_bn:
+				miu = self.bn.running_mean
+				var = self.bn.running_var
+				gamma = self.bn.weight
+				beta = self.bn.bias 
+				eps = self.bn.eps 
+				if gamma is None:
+					gamma = 1 
+				if beta is None:
+					beta = 0
+				weight = self.conv.weight
+				bias = self.conv.bias
+
+				if bias is not None:
+					b = gamma * (bias - miu) / torch.sqrt(eps + var) + beta 
+				else:
+					b = beta - (gamma * miu / torch.sqrt(eps + var))
+				bn_w = gamma / torch.sqrt(eps + var)
+				bn_w = bn_w.unsqueeze(-1).unsqueeze(-1).unsqueeze(-1)
+				w = bn_w * weight
+
+				res = {}
+				base_name = list(self.named_parameters())[0][0]
+				base_name = '.'.join(base_name.split('.')[:-1])
+				res[base_name+'.weight'] = w 
+				res[base_name+'.bias'] = b
+
+				if self.activation==PARAM_PRELU or self.activation==PARAM_PRELU1:
+					actw = self.act.weight
+					res['act.weight'] = actw
+				# print(res.keys())
+			else:
+				res = {}
+				for p in self.named_parameters():
+					res[p[0]] = p[1]
+				for p in self.named_buffers():
+					res[p[0]] = p[1]
 			L.record_params.append(res)
 		return x 
 
@@ -255,7 +357,7 @@ class Dense(Model):
 		self.batch_norm = batch_norm
 		self.activation = activation
 		if self.activation == PARAM_PRELU:
-			self.act = torch.nn.PReLU(num_parameters=outchn)
+			self.act = torch.nn.PReLU(num_parameters=outsize)
 		elif self.activation==PARAM_PRELU1:
 			self.act = torch.nn.PReLU(num_parameters=1)
 		if batch_norm:
@@ -277,7 +379,7 @@ class Dense(Model):
 		if record_flag:
 			# print(self._merge_bn)
 			# do record
-			if self._merge_bn:
+			if hasattr(self, 'bn') and self._merge_bn:
 				miu = self.bn.running_mean
 				var = self.bn.running_var
 				gamma = self.bn.weight
@@ -375,19 +477,19 @@ class ConvLSTM(Model):
 		return cell, h 
 
 class GraphConvLayer(Model):
-	def initialize(self, outsize, adj_mtx=None, adj_fn=None, usebias=True, activation=-1, batch_norm=False):
-		self.GCL = L.graphConvLayer(outsize, adj_mtx=adj_mtx, adj_fn=adj_fn, usebias=usebias)
+	def initialize(self, outsize, usebias=True, norm=True, activation=-1, batch_norm=False):
+		self.GCL = L.graphConvLayer(outsize, usebias=usebias, norm=norm)
 		self.batch_norm = batch_norm
 		self.activation = activation
 		if batch_norm:
-			self.bn = L.batch_norm()
+			self.bn = L.BatchNorm()
 		if self.activation == PARAM_PRELU:
-			self.act = torch.nn.PReLU(num_parameters=outchn)
+			self.act = torch.nn.PReLU(num_parameters=outsize)
 		elif self.activation==PARAM_PRELU1:
 			self.act = torch.nn.PReLU(num_parameters=1)
 
-	def forward(self, x):
-		x = self.GCL(x)
+	def forward(self, x, adj, affinity_grad=True):
+		x = self.GCL(x, adj, affinity_grad)
 		if self.batch_norm:
 			x = self.bn(x)
 		if self.activation==PARAM_PRELU or self.activation==PARAM_PRELU1:
