@@ -2,10 +2,12 @@ from . import DataReader
 from . import Layers 
 from . import Model
 from . import sul_tool
+from . import sulio 
 
 __all__ = [
 	'DataReader',
 	'Layers',
 	'Model',
 	'sul_tool'
+	'sulio'
 ]
\ No newline at end of file
